//!
//!
use super::{models, schema::role_resource_actions};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of role_resource_actions")
      .finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_resource: &models::NewRoleResourceAction,
  ) -> Result<models::RoleResourceAction, Error> {
    let inserted = diesel::insert_into(role_resource_actions::table)
      .values(new_resource)
      .get_result::<models::RoleResourceAction>(conn)?;

    Ok(inserted)
  }

  ///
  pub fn inserts(
    &self,
    conn: &diesel::PgConnection,
    new_resource_actions: &Vec<models::NewRoleResourceAction>,
  ) -> Result<Vec<models::RoleResourceAction>, Error> {
    let inserted = diesel::insert_into(role_resource_actions::table)
      .values(new_resource_actions)
      .get_results(conn)?;

    Ok(inserted)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    role_id: uuid::Uuid,
    resource_action_id: uuid::Uuid,
  ) -> Result<models::RoleResourceAction, Error> {
    use role_resource_actions::dsl;

    role_resource_actions::table
      .filter(
        dsl::role_id
          .eq(role_id)
          .and(dsl::resource_action_id.eq(resource_action_id)),
      )
      .first::<models::RoleResourceAction>(conn)
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = role_resource_actions::table.into_boxed();

    if let Some(sp) = find_all.role_id {
      q = q.filter(role_resource_actions::dsl::role_id.eq(sp));
    }
    if let Some(sp) = find_all.resource_action_id {
      q = q.filter(role_resource_actions::dsl::resource_action_id.eq(sp));
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<Vec<models::RoleResourceAction>, Error> {
    let mut q = role_resource_actions::table.into_boxed();

    if let Some(sp) = find_all.role_id {
      q = q.filter(role_resource_actions::dsl::role_id.eq(sp));
    }
    if let Some(sp) = find_all.resource_action_id {
      q = q.filter(role_resource_actions::dsl::resource_action_id.eq(sp));
    }

    if let Some(p) = find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = find_all.sorts {
      for s in orderbys {
        match s {
          bcr::models::pagination::Sort::ASC(property) => match property.as_str() {
            "role_id" => {
              q = q.order_by(role_resource_actions::role_id.asc());
            }
            "resource_action_id" => {
              q = q.order_by(role_resource_actions::resource_action_id.asc());
            }
            "created_at" => {
              q = q.order_by(role_resource_actions::created_at.asc());
            }
            _ => {}
          },
          bcr::models::pagination::Sort::DESC(property) => match property.as_str() {
            "role_id" => {
              q = q.order_by(role_resource_actions::role_id.desc());
            }
            "resource_action_id" => {
              q = q.order_by(role_resource_actions::resource_action_id.desc());
            }
            "created_at" => {
              q = q.order_by(role_resource_actions::created_at.desc());
            }

            _ => {}
          },
        };
      }
    }

    q.load::<models::RoleResourceAction>(conn)
  }

  ///
  pub fn delete(
    &self,
    conn: &diesel::PgConnection,
    role_id: uuid::Uuid,
    resource_action_id: uuid::Uuid,
  ) -> Result<u64, Error> {
    use role_resource_actions::dsl;

    diesel::delete(
      role_resource_actions::table.filter(
        dsl::role_id
          .eq(role_id)
          .and(dsl::resource_action_id.eq(resource_action_id)),
      ),
    )
    .execute(conn)
    .map(|c| c as u64)
  }

  ///
  pub fn delete_by_role_id(
    &self,
    conn: &diesel::PgConnection,
    role_id: uuid::Uuid,
  ) -> Result<u64, Error> {
    use role_resource_actions::dsl;

    diesel::delete(role_resource_actions::table.filter(dsl::role_id.eq(role_id)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
